// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WIN32_EVENTFAILABLE_H
#define HU_VARADIISTVAN_SCPL_IO_WIN32_EVENTFAILABLE_H
//------------------------------------------------------------------------------

#include "../Failable.h"

#include "EventFactory.h"

#include <memory>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Event;

//------------------------------------------------------------------------------

/**
 * A special subclass of Failable, the instances of which can be given
 * as a Failable target object to events. If some operation on an
 * event fails, the error code will be set on such an instance,
 * associated with the event.
 */
class EventFailable : public Failable
{
protected:
    /**
     * The event factory for this object.
     */
    std::unique_ptr<EventFactory> eventFactory;

protected:
    /**
     * Construct the object with the given event factory.
     */
    EventFailable(std::unique_ptr<EventFactory> eventFactory);

    /**
     * Create an event for this object using the event factory.
     */
    HANDLE createEvent();

    /**
     * Set the event represented by the given handle using the event factory.
     */
    bool setEvent(HANDLE handle);

    /**
     * Set the event represented by the given handle using the event factory.
     */
    bool resetEvent(HANDLE handle);

    /**
     * Destroy the event represented by the given handle using the event factory.
     */
    void destroyEvent(HANDLE handle);

    friend class Event;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline EventFailable::EventFailable(std::unique_ptr<EventFactory> eventFactory) :
    eventFactory(std::move(eventFactory))
{
}

//------------------------------------------------------------------------------

inline HANDLE EventFailable::createEvent()
{
    return eventFactory->createEvent();
}

//------------------------------------------------------------------------------

inline bool EventFailable::setEvent(HANDLE handle)
{
    return eventFactory->setEvent(handle);
}

//------------------------------------------------------------------------------

inline bool EventFailable::resetEvent(HANDLE handle)
{
    return eventFactory->resetEvent(handle);
}

//------------------------------------------------------------------------------

inline void EventFailable::destroyEvent(HANDLE handle)
{
    eventFactory->destroyEvent(handle);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_WIN32_EVENTFAILABLE_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
